/* ObjRexx  - grouping the grfxrexx functions. */
/* Using them to generate an img XHTML element */
-- Name: grfx_orexx.rex
-- Type: Object REXX Script
-- Author: Raphael Vanney
-- Resource: grfxrexx.zip, september 1995
-- Five REXX programs designed to fetch info from graphics files
-- BMP, GIF, PNG, JPEG, PCX.
-- Version: 20040826
-- Date: 26/08/2004
-- Copyright (C) Cie des Sables, 2004. All Rights Reserved.

/* if first character of returned string is a "(",
   an error occured and the returned string is a
   description of the error rather than that of the file.
   Otherwise, returns "width x height (color info)". */
   
/* grfxrexx modifications */
-- localisation, using: procedure expose debug .JPEG
-- modify GetPNGDesc; rename internal procedure from LitSegment to LitSegPNG
-- also the other LitSegment to LitSegJPEG

Parse Source system . programname
Parse Version level
Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
Call SysLoadFuncs -- load the RexxUtil functions

Call main

done:
  Call SysDropFuncs  -- drop all RexxUtil functions
  Exit -- the only exit.

main:
  long_path = 'D:\Genea\Genea files\TestGED\ImgFile.pcx'
  server_path = '/TestGED/'
  img_width = '000'; img_heigth ='000' -- default values (error or the referenced file does not exist)
  Debug=0  /* turn this off to avoid output from graphics functions */

	-- Parsing drive letter, path, search arguments from the user search specification
	-- The template combining string and positional patterns does not skip the matching data
	-- the . +1 drop the : character; this is the starting position of the relative positional pattern
	spos=Lastpos('\',long_path)-Pos(':',long_path)
	Parse var long_path drive ':' . +1 path +(spos) filename '.' ext
  ext = lower(ext)

  call SysFileTree long_path, 'exists', 'FO' -- is file there?
  file_exist = exists.0 > 0
	
/* Classic REXX */
  If file_exist Then
    Do
      Select
      When ext = 'jpg' Then img_info = GetJPEGDesc(long_path)
      When ext = 'bmp' Then img_info = GetBMPDesc(long_path)
      When ext = 'gif' Then img_info = GetGIFDesc(long_path)
      When ext = 'png' Then img_info = GetPNGDesc(long_path)
      When ext = 'pcx' Then img_info = GetPCXDesc(long_path)
      Otherwise
        img_info = '(no function to fetch info from' ext 'graphic file)'
      End /* Select */
    If Left(img_info,1) = '(' Then Nop
		Else Parse Var img_info img_width . img_heigth .  /* get values */
		End
  Else img_info = '(file' long_path 'not found on local disk)'
	Drop long_path exists.
  /* build the img XHTML element */
  img_xhtml = '<img src="'server_path||filename'.'ext'" width="'img_width'" height="'img_heigth'" alt="'filename'" />'
  Say img_xhtml
	
If Left(img_info,1) = '(' Then Say 'Error' img_info
Else
  Select
    When ext = 'jpg' Then
		  Do
			Say 'bits per sample     ' JPEG.bps
			Say 'image height        ' JPEG.height
			Say 'image width         ' JPEG.width
			Say 'number of components' JPEG.noc
			Say 'color information   ' JPEG.color
			End
    When ext = 'bmp' Then
		  Do
      Say "File size      "BMP.fsize
      Say "bfOffBits      "BMP.offbits
      Say "biSize         "BMP.isize
      Say "Image size     "BMP.width"x"BMP.height
      Say "# of planes    "BMP.planes
      Say "Bits/pixel     "BMP.bits
      Say "Compression    "BMP.comp
      Say "Size in bytes  "BMP.imgsize
			End
    When ext = 'gif' Then
		  Do
			Say 'bits per pixel  ' GIF.bits '(Log2(colors))'
			Say 'number of colors' GIF.colors
			Say 'background color' GIF.bkg
			Say 'image width     ' GIF.width
			Say 'image height    ' GIF.height
			End
    When ext = 'png' Then
		  Do
			Say 'width of image  ' PNG.width
      Say 'height of image ' PNG.height     
      Say 'bits per pixel  ' PNG.bits       
      Say 'color type      ' PNG.ctype
      Say 'compression type' PNG.comp
      Say 'filter type     ' PNG.filter
      Say 'interlace type  ' PNG.inter
      Say 'colors          ' PNG.info '(based on ctype & bits)'
			End
    When ext = 'pcx' Then
		  Do
			Say 'version of PaintBrush' PCX.version
			Say 'encoding method      ' PCX.comp '(should be 1=RLE)'
			Say 'bits/pixel/plan      ' PCX.bits
			Say 'Image coordinates    ' PCX.xmin '(usually xmin=ymin & xmin=0)'
			Say 'Image coordinates    ' PCX.ymin '(usually xmin=ymin & ymin=0)'
			Say 'Image coordinates    ' PCX.xmax
			Say 'Image coordinates    ' PCX.ymax
			Say 'image width          ' PCX.width '(xmax-xmin+1)'
			Say 'image height         ' PCX.height '(ymax-ymin+1)'
			Say '# of planes          ' PCX.planes
			Say 'color information    ' PCX.colors
			End
    Otherwise
      Nop
  End -- Select

Drop  BMP. GIF. PNG. JPEG. PCX.
/* End of Classic REXX */
return --main

/* Classic REXX */
GetJPEGDesc: procedure expose debug JPEG.
/* Get description for a JPEG file ; if first character of returned
   string is a "(", an error occured and the returned string is a
   description of the error rather than that of the file.
   Otherwise, returns "width x height (color info)".

   Extra info is available in JPEG. if no error occured :
   JPEG.bps         bits per sample
   JPEG.height      image height
   JPEG.width       image width
   JPEG.noc         number of components (I don't know either)
   JPEG.color       color information
*/
Parse Arg FName
Drop JPEG.

If FName="" Then Return "(must supply a filename)"

/* read file header */

Hdr=C2X(CharIn(FName, 1, 2))

If Debug Then Say "File header (SOI) : "Hdr
If Hdr<>"FFD8" Then Return "(not a JPEG file)"

NxtSeg=3
Do While (Seg.Type<>"D9") & (NxtSeg<>-1) & (JPEG.height="JPEG.HEIGHT")
     NxtSeg=LitSegJPEG(NxtSeg)
End

Drop Hdr NxtSeg Seg. Res FName

If JPEG.height<>"JPEG.HEIGHT"
Then Return JPEG.width||" x "||JPEG.height||" ("||JPEG.color||")" /* GetJPEGDesc */
Else Return "(size not found)" /* GetJPEGDesc */

LitSegJPEG:         /* reads a JPEG segment's header from the input file */
Arg SegPos

Seg.marker=C2X(CharIn(FName, SegPos))
If Seg.marker<>"FF" Then
Do
     If Debug Then Say "Invalid segment at "SegPos" ("Seg.marker")"
     Return -1
End
Seg.Type=C2X(CharIn(FName))
Res=SegPos+2                  /* position of next segment */
If (Seg.Type="01") | ((Seg.Type>="D0") & (Seg.Type<="D9")) Then
Do   /* these segments contain no data nor length info */
     Seg.Len=0
End
Else
Do
     Seg.Len=C2D(CharIn(FName, , 2))
End
Res=Res+Seg.Len

If (Seg.Type="C0") | (Seg.Type="C2") Then
Do
     /* start of frame 0 */
     JPEG.bps=C2D(CharIn(FName))             /* bits per sample */
     JPEG.height=C2D(CharIn(FName, , 2))
     JPEG.width=C2D(CharIn(FName, , 2))
     JPEG.noc=C2D(CharIn(FName))             /* number of components */
     Select
          When JPEG.noc=1 Then
               JPEG.color="grayscale"
          When JPEG.noc=3 Then
               JPEG.color="color YCbCr"
          When JPEG.noc=4 Then
               JPEG.color="color CMYK"
          Otherwise
               JPEG.color="?"
     End
End

If Debug Then Say "Segment at "D2X(SegPos, 4)", type "Seg.Type", length "Seg.Len

Return Res /* LitSegJPEG */

GetBMPDesc: procedure expose debug BMP.
/* Get description for a BMP file ; if first character of returned
   string is a "(", an error occured and the returned string is a
   description of the error rather than of the file.
   Normally returns "width x height (color info)".
   Extra info is available in BMP. if no error occured :
   (Note that the extra info is unaccurate for non-Windows BMP:s ; this
   is because I don't have a documentation for OS/2's BMP).

     BMP.fsize           file size
     BMP.offbits         ?
     BMP.isize           ?
     BMP.width           image width
     BMP.height          image height
     BMP.planes          number of planes
     BMP.bits            bits per pixel (per plane)
     BMP.comp            compression type
     BMP.imgsize         ?
*/

Parse Arg FName
Drop BMP.

If FName="" Then Return "(must supply a filename)"

/* read file header */

Hdr=C2D(Reverse(CharIn(FName, 1, 2)))             /* type */
BMP.fsize=C2D(Reverse(CharIn(FName, , 4)))        /* which size ? */
BMP.rsv=C2D(Reverse(CharIn(FName, , 4)))          /* should be 0 */

If (Hdr<>19778) | (BMP.rsv<>0) Then
Do
     Drop Hdr BMP.
     rc=CharOut(FName)
     Return "(not a BMP file)"
End

BMP.offbits=C2D(Reverse(CharIn(FName, , 4)))
BMP.isize=C2D(Reverse(CharIn(FName, , 4)))
BMP.width=C2D(Reverse(CharIn(FName, , 4)))        /* bitmap width */
BMP.height=C2D(Reverse(CharIn(FName, , 4)))       /* bitmap height */
BMP.planes=C2D(Reverse(CharIn(FName, , 2)))       /* # of planes */
BMP.bits=C2D(Reverse(CharIn(FName, , 2)))         /* bits per pel */
BMP.comp=C2D(Reverse(CharIn(FName, , 4)))         /* comp */
BMP.imgsize=C2D(Reverse(CharIn(FName, , 4)))      /* img size in bytes */

Bits=BMP.planes*BMP.bits
If Bits>=15 Then BMP.colors=Bits||"-bit true color"
            Else BMP.colors=2**Bits||" colors"
Drop Bits

If Debug Then
Do
     Say "Type           "Hdr
     Say "File size      "BMP.fsize
     Say "bfOffBits      "BMP.offbits
     Say "biSize         "BMP.isize
     Say "Image size     "BMP.width"x"BMP.height
     Say "# of planes    "BMP.planes
     Say "Bits/pixel     "BMP.bits
     Say "Compression    "BMP.comp
     Say "Size in bytes  "BMP.imgsize
End

rc=CharOut(FName)

Return BMP.width||" x "||BMP.height||" ("||BMP.colors||")"

GetGIFDesc: procedure expose debug GIF.
/* Get description for a GIF file ; if first character of returned
   string is a "(", an error occured and the returned string is a
   description of the error rather than of the file.
   Normally returns "width x height (colors)".
   Extra info is available in GIF. if no error occured :
     GIF.bits       bits per pixel (Log2(colors))
     GIF.colors     number of colors
     GIF.bkg        background color
     GIF.width      image width
     GIF.height     image height
*/

Parse Arg FName
Drop GIF.

If FName="" Then Return "(must supply a filename)"

/* read file header */

Hdr=CharIn(FName, 1, 6)                      /* should return "GIF8?a" */
If SubStr(Hdr, 1, 3)<>"GIF" Then
Do
     rc=CharOut(FName)
     Drop Hdr
     Return "(not a GIF file)"
End

CurPos=11

GIF.bits=C2D(BitAnd(CharIn(FName, CurPos), D2C(7)))+1
If Debug Then Say "GIF.bits="GIF.bits
GIF.colors=2 ** GIF.bits
GIF.bkg=C2D(CharIn(FName))

CurPos=CurPos+3+(3*GIF.colors)+5

GIF.width=C2D(Reverse(CharIn(FName, CurPos, 2)))
GIF.height=C2D(Reverse(CharIn(FName, , 2)))

Drop CurPos

rc=CharOut(FName)
Return GIF.width||" x "||GIF.height||" ("||GIF.colors||" colors)"

GetPCXDesc: procedure expose debug PCX.
/* Get description for a PCX file ; if first character of returned
   string is a "(", an error occured and the returned string is a
   description of the error rather than of the file.
   Normally returns "width x height (colors)".
   Extra info is available in PCX. if no error occured :

     PCX.version         version of PaintBrush
     PCX.comp            encoding method (should be 1=RLE)
     PCX.bits            bits/pixel/plan
     PCX.xmin            \
     PCX.ymin             \ Image coordinates ; usually xmin=ymin=0
     PCX.xmax             /
     PCX.ymax            /
     PCX.width           image width (xmax-xmin+1)
     PCX.height          image height (ymax-ymin+1)
     PCX.planes          # of planes
     PCX.colors          color information
*/

Parse Arg FName
Drop PCX.

If FName="" Then Return "(must supply a filename)"

/* read file header */

Hdr=C2D(CharIn(FName, 1))                    /* manufacturer */
PCX.version=C2D(CharIn(FName))               /* PaintBrush version */
PCX.comp=C2D(CharIn(FName))                  /* encoding (1=RLL) */
PCX.bits=C2D(CharIn(FName))                  /* bits per pixel */
PCX.xmin=C2D(Reverse(CharIn(FName, , 2)))
PCX.ymin=C2D(Reverse(CharIn(FName, , 2)))
PCX.xmax=C2D(Reverse(CharIn(FName, , 2)))
PCX.ymax=C2D(Reverse(CharIn(FName, , 2)))
PCX.width=PCX.xmax-PCX.xmin+1
PCX.height=PCX.ymax-PCX.ymin+1
PCX.planes=C2D(CharIn(FName, 66))            /* # of planes */

If (Hdr<>10) | (PCX.version>7) | (PCX.version<1) | ,
   (PCX.comp<0) | (PCX.comp>1) | (PCX.planes<0) | (PCX.planes>4) Then
Do
     Drop Hdr PCX.
     rc=CharOut(FName)
     Return "(not a PCX file)"
End
Drop Hdr

Bits=PCX.planes*PCX.bits
If Bits>=15 Then PCX.colors=Bits"-bit true color"
            Else PCX.colors=2**Bits" colors"
Drop Bits

If Debug Then
Do
     Say "PaintBrush version       "PCX.version
     Say "Encoding                 "PCX.comp
     Say "Bits/pixel               "PCX.bits
     Say "x0, y0, x1, y1           "PCX.xmin","PCX.ymin","PCX.xmax","PCX.ymax
     Say "Planes                   "PCX.planes
End

rc=CharOut(FName)

Return PCX.width||" x "||PCX.height||" ("||PCX.colors||")"

GetPNGDesc: procedure expose debug PNG.
/* Get description for a PNG file ; if first character of returned
   string is a "(", an error occured and the returned string is a
   description of the error rather than that of the file.
   Otherwise, returns "width x height (color info)".

   Extra info is available in PNG. if no error occured :
     PNG.width      width of image
     PNG.height     height of image
     PNG.bits       bits per pixel
     PNG.ctype      color type
     PNG.comp       compression type
     PNG.filter     filter type
     PNG.inter      interlace type
     PNG.info       text info regarding colors (based on ctype & bits)
*/
Parse Arg FName
Drop PNG.

If FName="" Then Return "(must supply a filename)"

/* read file header */

Hdr=CharIn(FName, 1, 8)
If Hdr<>(D2C(137)||"PNG"||D2C(13)||D2C(10)||D2C(26)||D2C(10)) Then
     Return "(not a PNG file)"

NxtSeg=9
Do While (Seg.Type<>"IEND") & (NxtSeg<>-1) & (PNG.height="PNG.HEIGHT")
     NxtSeg=LitSegPNG(NxtSeg)
End

Drop Hdr NxtSeg Seg. Res FName
rc=CharOut(FName)

If PNG.height<>"PNG.HEIGHT"
Then Return PNG.width||" x "||PNG.height||" ("||PNG.info||")"
Else Return "(size not found)"

LitSegPNG:         /* reads a PNG chunk's header from the input file */
Arg SegPos

Seg.len=C2D(CharIn(FName, SegPos, 4))   /* length of chunk's DATA */
Seg.Type=CharIn(FName, , 4)             /* chunk type */
Res=SegPos+12+Seg.len                   /* position of next chunk */

If Debug Then Say "Chunk at "D2X(SegPos, 4)", type "Seg.Type", length "Seg.len

If Seg.Type="IHDR" Then
Do
     /* header chunk */
     PNG.width=C2D(CharIn(FName, , 4))
     PNG.height=C2D(CharIn(FName, , 4))
     PNG.bits=C2D(CharIn(FName))
     PNG.ctype=C2D(CharIn(FName))
     PNG.comp=C2D(CharIn(FName))
     PNG.filter=C2D(CharIn(FName))
     PNG.inter=C2D(CharIn(FName))
     Select
          When PNG.ctype=0 Then
               PNG.info=2**PNG.bits||" gray levels"
          When PNG.ctype=2 Then
               PNG.info=3*PNG.bits||"-bit RGB color"
          When PNG.ctype=3 Then
               PNG.info=2**PNG.bits||" colors"
          When PNG.ctype=4 Then
               PNG.info=2**PNG.bits||" gray levels with alpha chanel"
          When PNG.ctype=6 Then
               PNG.info=3*PNG.bits||"-bit RGB color with alpha chanel"
          Otherwise
               PNG.info="unknow color type ("||PNG.ctype||")"
     End
End
Return Res /* LitSegPNG */

lower: return translate(arg(1), ,
'abcdefghijklmnopqrstuvwxyz',   ,
"ABCDEFGHIJKLMNOPQRSTUVWXYZ")
/* End of Classic REXX */
/*---------------------------- requires ------------------------------*/
::requires "OODPLAIN.cls"
